import numpy as np
import Nio as nio
import Ngl as ngl

def plot_ctt(ctt,ctt_msg_val,lat,lon,title=None):
  """
plot_ctt(ctt, ctt_msg_val, lat, lon)

This routine creates a contour/map plot of the cloud top temperature
variable read off an HDF-EOS file. It assumes the values have already
been unpacked.

ctt         - The array to plot
ctt_msg_val - The missing value
lat, lon    - The 2D lat,lon arrays 
title       - An optional title
  """

  wres          = ngl.Resources()
  wres.wkWidth  = 600
  wres.wkHeight = 600
  wks = ngl.open_wks("png","ctt_plot",wres)
  print("Graphical output is ctt_plot.png")

# Set some plot options
  res                   = ngl.Resources()

# Contour options
  res.cnFillOn          = True          # turn on contour fill
  res.cnLinesOn         = False         # turn off contour lines
  res.cnLineLabelsOn    = False         # turn off line labels
  res.cnFillMode        = "RasterFill"        # These two resources
  res.cnLevelSpacingF   = 2.5              # contour level spacing
  res.trGridType        = "TriangularMesh"    # can speed up plotting.
# res.cnFillPalette     = "WhViBlGrYeOrRe"

# Map options
  res.mpDataBaseVersion = "MediumRes"
  res.mpLimitMode       = "LatLon"
  res.mpMinLatF         = np.min(lat)-1
  res.mpMaxLatF         = np.max(lat)+1
  res.mpMinLonF         = np.min(lon)-1
  res.mpMaxLonF         = np.max(lon)+1
  res.mpGridAndLimbOn   = False

# Main Title
  if not title is None:
    res.tiMainString     = title

# Labelbar options
  res.lbLabelFontHeightF = 0.01

# Additional resources needed for putting contours on map
  res.sfXArray          = lon
  res.sfYArray          = lat
  res.sfMissingValueV   = ctt_msg_val

  plot = ngl.contour_map(wks,ctt,res)

  del wks  # close the PNG file just in case

