import numpy, Nio, Ngl

def plot_ts_Amon_subset_ngl(t,lat,lon,res=None):
  """
Creates a color contour plot over a map of the given variable, given
its lat/lon arrays. This routine is different than plot_ts_Amon_ngl,
in that it assumes the original variable was subsetted in lat/lon, and
hence it forces a zoom on the map.

plot_ts_Amon_subset_ngl(t,lat,lon)

t       - The variable to plot
lat,lon - The lat/lon arrays
  """

# Open file or window for graphics
  png_name = "plot_ts_Amon_subset"
  wres = Ngl.Resources()
  wres.wkWidth  = 500
  wres.wkHeight = 500
  wks = Ngl.open_wks("png",png_name,wres)

# Set some plot options
  if res is None:
    res = Ngl.Resources()

# Contour options
  res.cnFillOn          = True          # turn on contour fill
  res.cnLinesOn         = False         # turn off contour lines
  res.cnLineLabelsOn    = False         # turn off line labels
  res.cnFillPalette     = Ngl.read_colormap_file("MPL_RdYlBu")[::-1,:]

# Map options
  res.mpLimitMode       = "LatLon"
  res.mpMinLatF         = lat.min()
  res.mpMaxLatF         = lat.max()
  res.mpMinLonF         = lon.min()
  res.mpMaxLonF         = lon.max()
  res.mpDataBaseVersion = "MediumRes"

# Labelbar options
  res.lbOrientation     = "horizontal"  # default is "vertical"

# Main Title
  if hasattr(t,"long_name") and hasattr(t,"units"):
    res.tiMainString = getattr(res,"tiMainString","%s (%s)" % (t.long_name,t.units))

  res.tiMainFontHeightF = 0.02         # default is a big large

# Additional resources needed for putting contours on map
  res.sfXArray          = lon
  res.sfYArray          = lat

# Create the plot
  plot = Ngl.contour_map(wks,t,res)
  print("Created %s.png" % png_name)

# Main code
filename = "../Data/ts_Amon_CESM1-CAM5_historical_r1i1p1_185001-200512_partial.nc"
a   = Nio.open_file(filename)
ts  = a.variables["ts"]['time|i0 lat|0:60 lon|45:135']
lat = a.variables["lat"]['lat|0:60']
lon = a.variables["lon"]['lon|45:135']

plot_ts_Amon_subset_ngl(ts,lat,lon)

