# ## Examine an HDF-EOS file with missing data

# ### Import required modules

import Nio as nio
import numpy as np

filename="../Data/MOD06_L2.A2010031.1430.005.2010031221343.hdf"
f = nio.open_file(filename)
print(f)


# ### Read "Cloud_Top_Temperature" and print some information

ctt = f.variables["Cloud_Top_Temperature"]
print ctt.shape
print ctt
print "min/max CTT = %g / %g" % (np.min(ctt[:]) ,np.max(ctt[:]))

# ### Note the "scale_factor" and "add_offset" attributes

print "scale_factor",ctt.scale_factor
print "add_offset",ctt.add_offset


# ### Apply these attributes to unpack the data

cttf = ctt.scale_factor * (ctt[:] - ctt.add_offset)

print "min/max CTT = %g / %g" % (np.min(cttf) ,np.max(cttf))


# ### What happens if we take the average?

print "average = ",np.average(cttf)


# ### Note that cttf has a fill value

print ctt._FillValue         # This is a NioVariable
print cttf.get_fill_value()  # This is a MaskedArray


# ### You can test for a MaskedArray

print "Is cttf a MaskedArray?",isinstance(cttf, np.ma.MaskedArray)

# ### We have a masked array, so use "ma" module to take average

import numpy.ma as ma
print "average = ",ma.average(cttf)


# ### This will work too

print "average = ",cttf.mean()


# ### Create a color contour plot

from plot_ctt_ngl import *

# Read lat/lon off the file for plotting                                                  
lat = f.variables["Latitude"][:]
lon = f.variables["Longitude"][:]

title = "%s (%s)" % (ctt.hdf_name,ctt.units)
msg_val = float(ctt._FillValue)
plot_ctt(cttf,msg_val,lat,lon)

#from IPython.display import Image
#Image(filename='ctt_plot.png')

