import numpy, Nio, Ngl, os

filename = "../Data/wrfout_d03_2012-04-22_23_00_00_var_partial.nc"
f  = Nio.open_file(filename,"r")          # Open WRF output file
T  = f.variables["T"]                     # Read some variables
P  = f.variables["P"]
PB = f.variables["PB"]

TK  = T[:] + 300
PPB = P[:] + PB[:]
TC  = Ngl.wrf_tk(PPB, TK) - 273.16         # Convert to degC
print "TC after calculation:     %g / %g" % (numpy.min(TC[:]),numpy.max(TC[:]))

out_filename = "wrfout_new.nc"
if(os.path.exists(out_filename)):
  os.remove(out_filename)

fout = Nio.open_file(out_filename,"c")     # Open new file to write to
fout.title = "File for new WRF variable"   # Add a global file attribute

# Write dimensions of variable to file
for idim in range(len(T.dimensions)):
  fout.create_dimension(T.dimensions[idim],T.shape[idim])

# Define variable on file
new_vname = "TC"
fout.create_variable(new_vname,'d',T.dimensions)

# Write attributes of variable to file, changing two of them.
varAtts = T.__dict__.keys()
varAtts.sort()
for att in varAtts:
  if att == "description":
    value = "Temperature"
  elif att == "units":
    value = "degC"
  else:
    value = getattr(T,att)
  setattr(fout.variables[new_vname],att,value)

fout.variables[new_vname].assign_value(TC)      # Write data

fout.close()

# Reopen the file and read values back in
f = Nio.open_file(out_filename,"r")
tc = f.variables["TC"]
print "TC after reading back in: %g / %g" % (numpy.min(tc[:]),numpy.max(tc[:]))
