import numpy, Nio, Ngl, os

#---Read data
filename = "../Data/wrfout_d03_2012-04-22_23_00_00_var_partial.nc"
a  = Nio.open_file(filename)
T  = a.variables["T"][0,0,:,:]   # Read first time step (nlev x nlat x nlon)
P  = a.variables["P"][0,0,:,:]
PB = a.variables["PB"][0,0,:,:]

T  = T + 300
P  = P + PB
TK = Ngl.wrf_tk(P, T) - 273.16      # Convert to degC
print("Min/Max TK =",numpy.min(TK),"/",numpy.max(TK))

wks = Ngl.open_wks("x11","wrf_plot_TK_ngl_shp_outlines")

#---Set some plot options
res                   = Ngl.Resources()

res.nglDraw            = False     # Turn off draw and frame. Will do
res.nglFrame           = False     # this after we add shapefile outlines.

# Contour options
res.cnFillOn           = True               # turn on contour fill
res.cnLinesOn          = False              # turn off contour lines
res.cnLineLabelsOn     = False              # turn off line labels
res.cnFillMode         = "RasterFill"       # These two resources
res.trGridType         = "TriangularMesh"   # can speed up plotting.
res.cnFillPalette      = "ncl_default"      # change color map
res.lbLabelFontHeightF = 0.015              # default is a bit large
res.tiMainString       = "WRF data on native map projection"

# Add WRF native projection map options
Ngl.wrf_map_resources(filename,res)

res.mpOutlineOn       = False           # Will add shapefile outlines later

plot = Ngl.contour_map(wks,TK,res)

# Attach shapefile outlines to plot object.
shp_filename = "USA_adm/USA_adm2.shp"
if not os.path.exists(shp_filename):
  print "You need to download USA_adm shapefiles from gadm.org/country"
  import sys
  sys.exit(-1)

f   = Nio.open_file(shp_filename)
lon = numpy.ravel(f.variables["x"][:])
lat = numpy.ravel(f.variables["y"][:])

plres                  = Ngl.Resources()      # resources for polylines
plres.gsLineColor      = "navyblue"
plres.gsLineThicknessF = 2.0                  # default is 1.0
plres.gsSegments       = f.variables["segments"][:,0]

lnid = Ngl.add_polyline(wks, plot, lon, lat, plres)
Ngl.draw(plot)
Ngl.frame(wks)

Ngl.end()

